@@ -31,6 +31,11 @@ def comodularity_und(a1,a2):
 	same module) in at least one of these community structures.  Let f be the
 	set of nodes that are co-modular in both of these community structures.
 	The comodularity is |f|/|F|
+
+	This is actually very similar to the Jaccard index which turns out not
+	to be a terribly useful property. At high similarity a the variability
+	of information is better. It may be that the degenerate cross modularity
+	is even better though.
 	'''
 
 	ma,qa=bct.modularity_und(a1)
@@ -171,18 +176,23 @@ def entropic_similarity(a1,a2):
 	vi,_=bct.partition_distance(ma,mb)
 	return 1-vi
 
-def sample_degenerate_partitions(w, p_ceil=.10):
+def sample_degenerate_partitions(w, probtune_cap=.10, modularity_cutoff=.95):
+	ntries=0
 	while True:
 		init_ci,_ = bct.modularity_louvain_und_sign(w)
 		seed_ci, seed_q = bct.modularity_finetune_und_sign(w, ci=init_ci)	
 
-		p = np.random.random()*p_ceil
+		p = (np.random.random() * probtune_cap)
 		ci,q = bct.modularity_probtune_und_sign(w, ci=seed_ci, p=p)
-		if q > seed_q*.95:
-			print 'found one', q, seed_q
+		if q > (seed_q * modularity_cutoff):
+			print ('found a degenerate partition after %i tries with probtune ' 
+				'parameter %.3f: %.5f %.5f'%(ntries,p,q,seed_q))
+			ntries=0
 			yield ci,q
 		else:
-			print 'failed to find degenerate partition, trying again', q, seed_q
+			#print 'failed to find degenerate partition, trying again',q, seed_q
+			ntries+=1
+			
 
 def cross_modularity_degenerate(a1,a2,n=20):
 	a_degenerator = sample_degenerate_partitions(a1)
@@ -201,6 +211,43 @@ def cross_modularity_degenerate(a1,a2,n=20):
 
 	return accum/n
 
+def cross_modularity_degenerate_rate(a1,a2,omega,n=100):
+	a_degenerator = sample_degenerate_partitions(a1)
+	b_degenerator = sample_degenerate_partitions(a2)
+
+	#rate=0
+	for i in xrange(n):
+		a_ci,qa = a_degenerator.next()
+		b_ci,qb = b_degenerator.next()
+		_,qab = bct.modularity_und_sign(a1, b_ci)
+		_,qba = bct.modularity_und_sign(a2, a_ci)
+		eth_q = (qab+qba)/(qa+qb)
+		if eth_q > omega:
+			rate+=1
+		print 'trial %i, ethq=%.3f, estimated p-value %.3f for omega %.2f'%(
+			i,eth_q,(i+1-rate)/(i+1),omega)
+
+	return (n-rate)/n
+
+def cross_modularity_degenerate_raw(a1,a2,n=25,mc=.95,pc=.1):
+	a_degenerator = sample_degenerate_partitions(a1,modularity_cutoff=mc,
+		probtune_cap=pc)
+	b_degenerator = sample_degenerate_partitions(a2,modularity_cutoff=mc,
+		probtune_cap=pc)
+
+	raw = []
+	for i in xrange(n):
+		a_ci,qa = a_degenerator.next()
+		b_ci,qb = b_degenerator.next()
+		_,qab = bct.modularity_und_sign(a1, b_ci)
+		_,qba = bct.modularity_und_sign(a2, a_ci)
+		eth_q = (qab+qba)/(qa+qb)
+
+		print 'trial %i, ethq=%.3f'%(i,eth_q)
+		raw.append(eth_q)
+
+	return raw
+
 def nonparametric_similarity_test(similarity_metric,a1,a2,nshuff=1000):
 	true_similarity = similarity_metric(a1,a2)	
 
@@ -266,6 +313,10 @@ see Zalesky et al. (2012) "On the use of correlation as a measure of network
 # SMALL WORLD
 ###############################################################################
 
+from bct import breadthdist,charpath
+from bct import (clustering_coef_bd, clustering_coef_bu, clustering_coef_wd,
+	clustering_coef_wu)
+
 def small_world_bd(W):
 
 	'''
@@ -278,7 +329,8 @@ def small_world_bd(W):
 	output: s		small world coefficient
 	'''
 	cc = clustering_coef_bd(W)
-	_lambda,_,_,_,_ = charpath(W)
+	_,dists = breadthdist(W)
+	_lambda,_,_,_,_ = charpath(dists)
 	return np.mean(cc)/_lambda
 
 def small_world_bu(W):
@@ -292,7 +344,8 @@ def small_world_bu(W):
 	output: s		small world coefficient
 	'''
 	cc = clustering_coef_bu(W)
-	_lambda,_,_,_,_ = charpath(W)
+	_,dists = breadthdist(W)
+	_lambda,_,_,_,_ = charpath(dists)
 	return np.mean(cc)/_lambda
 
 def small_world_wd(W):
@@ -306,7 +359,8 @@ def small_world_wd(W):
 	output: s		small world coefficient
 	'''
 	cc = clustering_coef_wd(W)
-	_lambda,_,_,_,_ = charpath(W)
+	_,dists = breadthdist(W)
+	_lambda,_,_,_,_ = charpath(dists)
 	return np.mean(cc)/_lambda
 
 def small_world_wu(W):
@@ -320,5 +374,6 @@ def small_world_wu(W):
 	output: s		small world coefficient
 	'''
 	cc = clustering_coef_wu(W)
-	_lambda,_,_,_,_ = charpath(W)
+	_,dists = breadthdist(W)
+	_lambda,_,_,_,_ = charpath(dists)
 	return np.mean(cc)/_lambda
